@@ -32,6 +32,8 @@
          basic/1,
          call/1,
          meta/1,
+         ms_enable_flags/1,
+         return_to/1,
          destroy/1,
          negative/1,
          error_info/1,
@@ -47,7 +49,7 @@
 -define(line,void).
 -endif.
 
--export([foo/0, exported/1]).
+-export([foo/0, exported/1, middle/1, bottom/1]).
 
 suite() ->
     [{ct_hooks,[ts_install_cth]},
@@ -58,6 +60,7 @@ all() ->
      basic,
      call,
      meta,
+     ms_enable_flags,
      on_load,
      trace_info_on_load,
      procs,
@@ -65,6 +68,7 @@ all() ->
      test_set_on_first_spawn,
      test_set_on_link,
      test_set_on_first_link,
+     return_to,
      destroy,
      negative,
      error_info,
@@ -380,7 +384,7 @@ set_on_spawn2(S1, Tracer1, Opts1, S2, Tracer2, Opts2) ->
         {Tracer1, {trace, GrandChild, exit, die}}],
 
        [{Tracer2, {trace, GrandChild, spawned, Child, '_'}},
-        {Tracer2,{trace, '_', exit, die}}]
+        {Tracer2, {trace, GrandChild, exit, die}}]
       }),
 
     1 = erlang_trace(S1, self(), false, [procs, set_on_spawn | Opts1]),
@@ -1002,12 +1006,11 @@ basic_do2(S1, Tracer1, Opts1, S2, Tracer2, Opts2) ->
     register(RegName, Tracee),
     unregister(RegName),
 
-    receive_unsorted(
-      [{Tracer1, {trace, Tracee, register, RegName}},
-       {Tracer2, {trace, Tracee, register, RegName}}]),
-    receive_unsorted(
-      [{Tracer1, {trace, Tracee, unregister, RegName}},
-       {Tracer2, {trace, Tracee, unregister, RegName}}]),
+    receive_parallel({[{Tracer1, {trace, Tracee, register, RegName}},
+                       {Tracer1, {trace, Tracee, unregister, RegName}}],
+
+                      [{Tracer2, {trace, Tracee, register, RegName}},
+                       {Tracer2, {trace, Tracee, unregister, RegName}}]}),
 
     1 = erlang_trace(S1, self(), false, [procs | Opts1]),
 
@@ -1199,6 +1202,223 @@ meta_do(S1, Tracer1, S2, Tracer2) ->
 
     ok.
 
+%% Test that enable trace flags with match spec on untraced process
+%% uses session tracer and not tracer of current process.
+ms_enable_flags(_Config) ->
+    Tester = self(),
+    Dummy = spawn_link(fun() -> receive die -> ok end end),
+    Tracer1 = spawn_link(fun() -> tracer("Tracer1",Tester) end),
+    S1 = trace:session_create(session1, Tracer1, []),
+
+    %% Test enable trace flag on current process
+    Fun = fun(EnableSend, DisableSend) ->
+                  trace:function(S1, {?MODULE,foo,0},
+                                 [{'_', [], [EnableSend]}],
+                                 [meta]),
+
+                  foo(),
+                  {Tracer1, {trace_ts, Tester, call, {?MODULE,foo,[]}, {_,_,_}}}
+                      = receive_any(),
+
+                  {flags, [send]} = trace:info(S1, Tester, flags),
+                  Dummy ! message,
+                  {Tracer1, {trace, Tester, send, message, Dummy}} = receive_any(),
+
+                  trace:function(S1, {?MODULE,foo,0},
+                                 [{'_', [], [DisableSend]}],
+                                 [meta]),
+                  Dummy ! message,
+                  {Tracer1, {trace, Tester, send, message, Dummy}} = receive_any(),
+                  foo(),
+                  {Tracer1, {trace_ts, Tester, call, {?MODULE,foo,[]}, {_,_,_}}}
+                      = receive_any(),
+                  {flags, []} = trace:info(S1, Tester, flags),
+                  timeout = receive_nothing(),
+                  ok
+          end,
+    Fun({trace, [], [send]}, {trace, [send], []}),
+    Fun({enable_trace, send}, {disable_trace, send}),
+
+    %% Test enable trace flag on other process
+    Other = spawn_link(fun() -> receive die -> ok end end),
+    Fun2 = fun(EnableRecv, DisableRecv) ->
+                   trace:function(S1, {?MODULE,foo,0},
+                                  [{'_', [], [EnableRecv]}],
+                                  [meta]),
+
+                  foo(),
+                  {Tracer1, {trace_ts, Tester, call, {?MODULE,foo,[]}, {_,_,_}}}
+                       = receive_any(),
+
+                  {flags, ['receive']} = trace:info(S1, Other, flags),
+                  Other ! message,
+                  {Tracer1, {trace, Other, 'receive', message}} = receive_any(),
+
+                  trace:function(S1, {?MODULE,foo,0},
+                                 [{'_', [], [DisableRecv]}],
+                                 [meta]),
+                  Other ! message,
+                  {Tracer1, {trace, Other, 'receive', message}} = receive_any(),
+                  foo(),
+                  {Tracer1, {trace_ts, Tester, call, {?MODULE,foo,[]}, {_,_,_}}}
+                       = receive_any(),
+                  {flags, []} = trace:info(S1, Other, flags),
+                  timeout = receive_nothing(),
+
+                  ok
+          end,
+    Fun2({trace, Other, [], ['receive']}, {trace, Other, ['receive'], []}),
+    Fun2({enable_trace, Other, 'receive'}, {disable_trace, Other, 'receive'}),
+
+    ok.
+
+
+return_to(_Config) ->
+    %%put(display, true),  %% To get some usable debug printouts
+
+    Tester = self(),
+    Tracer1 = spawn_link(fun() -> tracer("Tracer1",Tester,get(display)) end),
+    Tracer2 = spawn_link(fun() -> tracer("Tracer2",Tester,get(display)) end),
+
+    Tracee = self(),
+    S1 = trace:session_create(session1, Tracer1, []),
+    S2 = trace:session_create(session2, Tracer2, []),
+
+    1 = trace:process(S1, Tracee, true, [call, return_to, arity]),
+    1 = trace:process(S2, Tracee, true, [call, return_to, arity]),
+
+    [begin
+         Traced = [{S1, Tracer1, Funcs1},
+                   {S2, Tracer2, Funcs2}],
+
+         %% Set up tracing for all sessions
+         [begin
+              trace:function(Session, {?MODULE,'_','_'}, false, [local]),
+              [begin
+                   io_format("trace:function(~p) for tracer ~p\n", [Func, Tracer]),
+                   1 = trace:function(Session, {?MODULE,Func,'_'}, true, [local])
+               end
+               || Func <- TracedFuncs]
+          end
+          || {Session, Tracer, TracedFuncs} <- Traced],
+
+         %% Execute test with both local and external calls
+         [return_to_do(MiddleCall, BottomCall, Traced)
+          || MiddleCall <- [local, extern],
+             BottomCall <- [local, extern]]
+     end
+     || Funcs1 <- [[bottom], [middle], [middle,bottom]],
+        Funcs2 <- [[bottom], [middle], [middle,bottom]]
+    ],
+
+    true = trace:session_destroy(S1),
+    ok.
+
+return_to_do(MiddleCall, BottomCall, Traced) ->
+    Tracee = self(),
+    Script = [{[{'catch',MiddleCall}, {body,BottomCall}, exception],
+               #{[bottom] => [{call, bottom}, {return_to, top}],
+                 [middle] => [{call, middle}, {return_to, top}],
+                 [middle,bottom] => [{call, middle}, {call, bottom}, {return_to,top}]
+                }},
+
+              {[{'catch',MiddleCall}, {'catch',BottomCall}, exception],
+               #{[bottom] => [{call, bottom}, {return_to, middle}],
+                 [middle] => [{call, middle}, {return_to, top}],
+                 [middle,bottom] => [{call, middle}, {call, bottom}, {return_to,middle}, {return_to,top}]
+                }},
+
+              {[{'catch',MiddleCall}, {tail,BottomCall}, exception],
+               #{[bottom] => [{call, bottom}, {return_to, top}],
+                 [middle] => [{call, middle}, {return_to, top}],
+                 [middle,bottom] => [{call, middle}, {call, bottom}, {return_to,top}]
+                }},
+
+              {[{'catch',MiddleCall},  {body,BottomCall}, return],
+               #{[bottom] => [{call, bottom}, {return_to, middle}],
+                 [middle] => [{call, middle}, {return_to, top}],
+                 [middle,bottom] => [{call, middle}, {call, bottom}, {return_to,middle}, {return_to,top}]
+                }},
+
+              {[{'catch',MiddleCall}, {'catch',BottomCall}, return],
+               #{[bottom] => [{call,bottom}, {return_to,middle}],
+                 [middle] => [{call, middle}, {return_to, top}],
+                 [middle,bottom] => [{call, middle}, {call, bottom}, {return_to,middle}, {return_to,top}]
+                }},
+
+              {[{'catch',MiddleCall}, {tail,BottomCall}, return],
+               #{[bottom] => [{call,bottom}, {return_to,top}],
+                 [middle] => [{call, middle}, {return_to, top}],
+                 [middle,bottom] => [{call, middle}, {call, bottom}, {return_to,top}]
+                }}
+             ],
+
+    [begin
+         %% Make the call sequence
+         io_format("CallSequence = ~p\n", [CallSequence]),
+         top(CallSequence),
+
+         %% Construct expected trace messages
+         Exp = [[{Tracer, {trace, Tracee, CallOrReturnTo, {?MODULE, Func, 1}}}
+                 || {CallOrReturnTo, Func} <- maps:get(TracedFuncs, TraceMap, [])]
+                || {_Session, Tracer, TracedFuncs} <- Traced],
+
+
+         io_format("Exp = ~p\n", [Exp]),
+         receive_parallel_list(Exp)
+     end
+     || {CallSequence, TraceMap} <- Script],
+    ok.
+
+top([{'catch',local} | T]) ->
+    display("top(catch local)"),
+    [(catch middle(T)) | 1];
+top([{'catch',extern} | T]) ->
+    display("top(catch extern)"),
+    [(catch ?MODULE:middle(T)) | 1].
+
+middle([{body,local} | T]) ->
+    display("middle(local)"),
+    [bottom(T) | 1];
+middle([{body,extern} | T]) ->
+    display("middle(extern)"),
+    [?MODULE:bottom(T) | 1];
+middle([{'catch',local} | T]) ->
+    display("middle(catch local)"),
+    [(catch bottom(T)) | 1];
+middle([{'catch',extern} | T]) ->
+    display("middle(catch extern)"),
+    [(catch ?MODULE:bottom(T)) | 1];
+middle([{tail,local} | T]) ->
+    display("middle(tail local)"),
+    bottom(T);
+middle([{tail,extern} | T]) ->
+    display("middle(tail extern)"),
+    ?MODULE:bottom(T).
+
+bottom([return | T]) ->
+    display("bottom(return)"),
+    T;
+bottom([exception]) ->
+    display("bottom(exception)"),
+    error(exception).
+
+display(Term) ->
+    case get(display) of
+        true ->
+            erlang:display(Term);
+        _ ->
+            true
+    end.
+
+io_format(Frmt, List) ->
+    case get(display) of
+        true ->
+            io:format(Frmt, List);
+        _ ->
+            ok
+    end.
+
 destroy(_Config) ->
     Name = ?MODULE,
     {_,SName1}=S1 = trace:session_create(Name, self(), []),
@@ -1396,11 +1616,22 @@ local(_) ->
     ok.
 
 tracer(Name, Tester) ->
+    tracer(Name, Tester, true).
+
+tracer(Name, Tester, Display) ->
+    case Display of
+        true -> put(display,true);
+        _ -> ok
+    end,
+    tracer_loop(Name, Tester).
+
+
+tracer_loop(Name, Tester) ->
     receive M ->
-            io:format("~p ~p got message: ~p\n", [Name, self(), M]),
+            io_format("~p ~p got message: ~p\n", [Name, self(), M]),
             Tester ! {self(), M}
     end,
-    tracer(Name, Tester).
+    tracer_loop(Name, Tester).
 
 
 receive_any() ->
@@ -1440,6 +1671,11 @@ receive_parallel(M, Tuple, 0) ->
     io:format("Got message:\n~p\n", [M]),
     ct:fail("Unexpected messages: ~p", [M]).
 
+%% Same as receive_parallel/1 but accepts a *list* of message lists
+%% and the message lists are allowed to be empty meaning no expected messages.
+receive_parallel_list(List0) ->
+    List1 = lists:filter(fun(E) -> E =/= [] end, List0),
+    receive_parallel(list_to_tuple(List1)).
 
 receive_unsorted(Expect) ->
     receive_unsorted(Expect, length(Expect)).
