@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1998-2023. All Rights Reserved.
+%% Copyright Ericsson AB 1998-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -609,7 +609,20 @@ do_shutdown_error(Config, Addr) ->
     ?P("create listen socket"),
     {ok, L} = gen_tcp:listen(0, ?INET_BACKEND_OPTS(Config) ++ [{ip, Addr}]),
     ?P("shutdown socket (with How = read_write)"),
-    {error, enotconn} = gen_tcp:shutdown(L, read_write),
+    case gen_tcp:shutdown(L, read_write) of
+        {error, enotconn} ->
+            ok;
+        ok -> % On some platforms this can happen...Linux...
+            case os:type() of
+                {unix, linux} ->
+                    ?P("unexpected shutdown success - can happen on linux"),
+                    ok;
+                _ ->
+                    exit(unexpected_success)
+            end;
+        {error, Reason} ->
+            exit({unexpected_shutdown_error, Reason})
+    end,
     ?P("close socket"),
     ok = gen_tcp:close(L),
     ?P("shutdown socket again (with How = read_write)"),
