@@ -42,8 +42,8 @@ get_context(_Config) ->
     {term} = edlin_context:get_context(lists:reverse("h(file,open), h(file")),
     {term} = edlin_context:get_context(lists:reverse("h(file,open), h(file,open")),
     {term, [{call, "h(file,open)"}], {call, "h(file,open)"}} = edlin_context:get_context(lists:reverse("h(file,open), h(file,open)")),
-    {function} = edlin_context:get_context(lists:reverse("file:")),
-    {function} = edlin_context:get_context(lists:reverse("file:open")),
+    {function, "file"} = edlin_context:get_context(lists:reverse("file:")),
+    {function, "file"} = edlin_context:get_context(lists:reverse("file:open")),
     {function, "file", "open", [], [], []} = edlin_context:get_context(lists:reverse("file:open(")),
     {string} = edlin_context:get_context(lists:reverse("file:open(\"")),
     {string} = edlin_context:get_context(lists:reverse("file:open(\"/")),
@@ -188,4 +188,10 @@ get_context(_Config) ->
     {term,[],{float,"-1.2"}} = edlin_context:get_context(lists:reverse("-1.2")),
     {term,[],{tuple, "{hej, svej}"}} = edlin_context:get_context(lists:reverse("begin {hej, svej}")),
     {term,[],[]} = edlin_context:get_context(lists:reverse("begin {hej, svej} = {")),
+    {fun_} = edlin_context:get_context(lists:reverse("fun(")),
+    {fun_} = edlin_context:get_context(lists:reverse("maps:map(fun(")),
+    {term} = edlin_context:get_context(lists:reverse("/")),
+    {term} = edlin_context:get_context(lists:reverse("Foo/")),
+    {fun_, "user_defined", "a"} = edlin_context:get_context(lists:reverse("a/")),
+    {term} = edlin_context:get_context(lists:reverse("Foo(")),
     ok.
\ No newline at end of file
