@@ -62,8 +62,6 @@
 
 -module(wx).
 -moduledoc """
-A port of wxWidgets.
-
 A port of [wxWidgets](http://www.wxwidgets.org/).
 
 This is the base api of [wxWidgets](http://www.wxwidgets.org/). This module
@@ -174,14 +172,12 @@ Global (classless) functions are located in the wx_misc module.
 -type wx_enum() :: integer().      %% Constant defined in wx.hrl
 -type wx_wxHtmlLinkInfo() :: #wxHtmlLinkInfo{}.
 
--doc "parent_class(X1) -> term()".
--spec parent_class(wx_object()) -> boolean().
+-doc "".
+-spec parent_class(Wx_ref :: wx_object()) -> boolean().
 parent_class(_) -> true. %% Let the null pointers be sent down.
 
 %% @doc Starts a wx server.
 -doc """
-new() -> wx_object()
-
 Starts a wx server.
 """.
 -spec new() -> wx_object().
@@ -194,10 +190,10 @@ new() ->
 %% be suppressed. The latter can be used as a silent test of whether
 %% wx is properly installed or not.
 -doc """
-new(Options::[Option]) -> wx_object()
+Starts a wx server.
 
-Starts a wx server. Option may be \{debug, Level\}, see debug/1. Or
-\{silent_start, Bool\}, which causes error messages at startup to be suppressed.
+Option may be `{debug, Level}`, see `debug/1`. Or `{silent_start, Bool}`,
+which causes error messages at startup to be suppressed.
 The latter can be used as a silent test of whether wx is properly installed or
 not.
 """.
@@ -214,8 +210,6 @@ new(Options) when is_list(Options) ->
 
 %% @doc Stops a wx server.
 -doc """
-destroy() -> ok
-
 Stops a wx server.
 """.
 -spec destroy() -> 'ok'.
@@ -228,8 +222,6 @@ destroy() ->
 %% Can be sent to other processes to allow them use this process wx environment.
 %% @see set_env/1
 -doc """
-get_env() -> wx_env()
-
 Gets this process's current wx environment. Can be sent to other processes to
 allow them use this process wx environment.
 
@@ -245,12 +237,10 @@ get_env() ->
 %% @doc Sets the process wx environment, allows this process to use
 %% another process wx environment.
 -doc """
-set_env(Wx_env::wx_env()) -> ok
-
 Sets the process wx environment, allows this process to use another process wx
 environment.
 """.
--spec set_env(wx_env()) -> 'ok'.
+-spec set_env(Wx_env :: wx_env()) -> 'ok'.
 set_env(#wx_env{sv=Pid} = Env) ->
     put(?WXE_IDENTIFIER, Env),
     %%    put(opengl_port, Port),
@@ -272,8 +262,6 @@ set_env(#wx_env{sv=Pid} = Env) ->
 %% The call always returns ok but will have sent any already received
 %% events to the calling process.
 -doc """
-subscribe_events() -> ok
-
 Adds the calling process to the list of of processes that are listening to wx
 application events.
 
@@ -296,8 +284,6 @@ subscribe_events() ->
 
 %% @doc Returns the null object
 -doc """
-null() -> wx_object()
-
 Returns the null object
 """.
 -spec null() -> wx_object().
@@ -306,29 +292,23 @@ null() ->
 
 %% @doc Returns true if object is null, false otherwise
 -doc """
-is_null(Wx_ref::wx_object()) -> boolean()
-
 Returns true if object is null, false otherwise
 """.
--spec is_null(wx_object()) -> boolean().
+-spec is_null(Wx_ref :: wx_object()) -> boolean().
 is_null(#wx_ref{ref=NULL}) -> NULL =:= 0.
 
 %% @doc Returns true if both arguments references the same object, false otherwise
 -doc """
-equal(Wx_ref::wx_object(), X2::wx_object()) -> boolean()
-
 Returns true if both arguments references the same object, false otherwise
 """.
--spec equal(wx_object(), wx_object()) -> boolean().
+-spec equal(Ref1 :: wx_object(), Ref2 :: wx_object()) -> boolean().
 equal(#wx_ref{ref=Ref1}, #wx_ref{ref=Ref2}) -> Ref1 =:= Ref2.
 
 %% @doc Returns the object type
 -doc """
-getObjectType(Wx_ref::wx_object()) -> atom()
-
 Returns the object type
 """.
--spec getObjectType(wx_object()) -> atom().
+-spec getObjectType(Wx_ref :: wx_object()) -> atom().
 getObjectType(#wx_ref{type=Type}) ->
     Type.
 
@@ -336,8 +316,6 @@ getObjectType(#wx_ref{type=Type}) ->
 %%  It is needed when using functions like wxWindow:findWindow/2, which
 %%  returns a generic wxObject type.
 -doc """
-typeCast(Old::wx_object(), NewType::atom()) -> wx_object()
-
 Casts the object to class NewType. It is needed when using functions like
 wxWindow:findWindow/2, which returns a generic wxObject type.
 """.
@@ -355,8 +333,6 @@ typeCast(Old=#wx_ref{}, NewType) when is_atom(NewType) ->
 %% @see foldl/3
 %% @see foldr/3
 -doc """
-batch(Fun::function()) -> term()
-
 Batches all `wx` commands used in the fun. Improves performance of the command
 processing by grabbing the wxWidgets thread so that no event processing will be
 done before the complete batch of commands is invoked.
@@ -377,8 +353,6 @@ batch(Fun) ->
 
 %% @doc Behaves like {@link //stdlib/lists:foreach/2} but batches wx commands. See {@link batch/1}.
 -doc """
-foreach(Fun::function(), List::list()) -> ok
-
 Behaves like `lists:foreach/2` but batches wx commands. See `batch/1`.
 """.
 -spec foreach(function(), list()) -> 'ok'.
@@ -395,8 +369,6 @@ foreach(Fun, List) ->
 
 %% @doc Behaves like {@link //stdlib/lists:map/2} but batches wx commands. See {@link batch/1}.
 -doc """
-map(Fun::function(), List::list()) -> list()
-
 Behaves like `lists:map/2` but batches wx commands. See `batch/1`.
 """.
 -spec map(function(), list()) -> list().
@@ -413,8 +385,6 @@ map(Fun, List) ->
 
 %% @doc Behaves like {@link //stdlib/lists:foldl/3} but batches wx commands. See {@link batch/1}.
 -doc """
-foldl(Fun::function(), Acc::term(), List::list()) -> term()
-
 Behaves like `lists:foldl/3` but batches wx commands. See `batch/1`.
 """.
 -spec foldl(function(), term(), list()) -> term().
@@ -431,8 +401,6 @@ foldl(Fun, Acc, List) ->
 
 %% @doc Behaves like {@link //stdlib/lists:foldr/3} but batches wx commands. See {@link batch/1}.
 -doc """
-foldr(Fun::function(), Acc::term(), List::list()) -> term()
-
 Behaves like `lists:foldr/3` but batches wx commands. See `batch/1`.
 """.
 -spec foldr(function(), term(), list()) -> term().
@@ -458,8 +426,6 @@ foldr(Fun, Acc, List) ->
 %% Use it carefully.
 
 -doc """
-create_memory(Size::integer()) -> wx_memory()
-
 Creates a memory area (of Size in bytes) which can be used by an external
 library (i.e. opengl). It is up to the client to keep a reference to this object
 so it does not get garbage collected by erlang while still in use by the
@@ -468,7 +434,7 @@ external library.
 This is far from erlang's intentional usage and can crash the erlang emulator.
 Use it carefully.
 """.
--spec create_memory(integer()) -> wx_memory().
+-spec create_memory(Size :: integer()) -> wx_memory().
 create_memory(Size) when Size > ?MIN_BIN_SIZE ->
     #wx_mem{bin = <<0:(Size*8)>>, size = Size};
 create_memory(Size) ->
@@ -476,11 +442,9 @@ create_memory(Size) ->
 
 %% @doc Returns the memory area as a binary.
 -doc """
-get_memory_bin(Wx_mem::wx_memory()) -> binary()
-
 Returns the memory area as a binary.
 """.
--spec get_memory_bin(wx_memory()) -> binary().
+-spec get_memory_bin(Wx_mem :: wx_memory()) -> binary().
 get_memory_bin(#wx_mem{bin=Bin, size=Size}) when Size > ?MIN_BIN_SIZE ->
     Bin;
 get_memory_bin(#wx_mem{bin=Bin, size=Size}) ->
@@ -490,12 +454,10 @@ get_memory_bin(#wx_mem{bin=Bin, size=Size}) ->
 %% @doc Saves the memory from deletion until release_memory/1 is called.
 %% If release_memory/1 is not called the memory will not be garbage collected.
 -doc """
-retain_memory(Wx_mem::wx_memory()) -> ok
-
-Saves the memory from deletion until release_memory/1 is called. If
-release_memory/1 is not called the memory will not be garbage collected.
+Saves the memory from deletion until `release_memory/1` is called. If
+`release_memory/1` is not called the memory will not be garbage collected.
 """.
--spec retain_memory(wx_memory()) -> 'ok'.
+-spec retain_memory(Wx_mem :: wx_memory()) -> 'ok'.
 retain_memory(#wx_mem{}=Mem) ->
     case get(Mem) of
         {Mem, N} -> put(Mem, N+1);
@@ -509,8 +471,8 @@ retain_memory(Bin) when is_binary(Bin) ->
     end,
     retain_memory(#wx_mem{bin=Bin, size=byte_size(Bin)}).
 
--doc "release_memory(Wx_mem::wx_memory()) -> ok".
--spec release_memory(wx_memory()) -> 'ok'.
+-doc "Releases the memory retained by `retain_memory/1`".
+-spec release_memory(Wx_mem :: wx_memory()) -> 'ok'.
 release_memory(#wx_mem{}=Mem) ->
     case erase(Mem) of
         1 -> ok;
@@ -524,7 +486,7 @@ release_memory(Bin) when is_binary(Bin) ->
 %% each call is printed on console. If Level is 'driver' each allocated
 %% object and deletion is printed on the console.
 -doc """
-debug(Debug::Level | [Level]) -> ok
+debug(Level)
 
 Sets debug level. If debug level is 'verbose' or 'trace' each call is printed on
 console. If Level is 'driver' each allocated object and deletion is printed on
@@ -571,8 +533,6 @@ set_debug(Level) when is_integer(Level) ->
 
 %% @doc Starts a Wx demo if examples directory exists and is compiled
 -doc """
-demo() -> ok | {error, atom()}
-
 Starts a Wx demo if examples directory exists and is compiled
 """.
 -spec demo() -> 'ok' | {'error', atom()}.
